"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = loadConfig;
exports.overrideKeysFromCLI = overrideKeysFromCLI;
exports.readConfigFile = readConfigFile;
exports.saveConfigToFile = saveConfigToFile;
exports.validateConfig = validateConfig;
const lodash_1 = __importDefault(require("lodash"));
const node_fs_1 = require("node:fs");
const node_path_1 = __importDefault(require("node:path"));
const constants_1 = require("./constants");
const inquirer_1 = require("./inquirer");
const logger_1 = require("./logger");
const schema_1 = require("./schema");
/**
 * Loads the application configuration from the specified configuration file and command-line arguments.
 *
 * This function first checks if the global `config` object is already set. If it is, it simply returns the existing configuration.
 *
 * If the `config` object is not set, it reads the configuration file specified by the `config-file` command-line argument, or the default configuration file name if the argument is not provided. It then validates the configuration data and sets any missing keys to their default values.
 *
 * Finally, it overrides the configuration object with any values from the command-line arguments using the `overrideKeysFromCLI` function, and sets the global `config` object to the updated configuration.
 *
 * @param argv - The command-line arguments.
 */
function loadConfig(argv) {
    if (globalThis.config)
        return;
    const configFile = String(argv["config-file"]);
    const config = readConfigFile(configFile);
    validateConfig(config);
    const updatedConfig = setMissingKeysToDefault(config);
    globalThis.config = overrideKeysFromCLI(updatedConfig, argv);
}
/**
 * Overrides the configuration object with values from the command-line arguments.
 *
 * This function takes the existing configuration object and the command-line arguments,
 * and updates the configuration object with any values from the command-line arguments.
 * The updated configuration object is then returned.
 *
 * @param config - The existing configuration object.
 * @param argv - The command-line arguments.
 * @returns The updated configuration object.
 */
function overrideKeysFromCLI(config, argv) {
    if (argv.debug)
        config.debug = Boolean(argv.debug);
    if (argv.destination)
        config.destination = String(argv.destination);
    if (argv.header)
        config.header = String(argv.header);
    if (Array.isArray(argv.excludes))
        config.excludes = argv.excludes;
    if (argv.force)
        config.force = Boolean(argv.force);
    if (argv.silent)
        config.logOptions.silent = Boolean(argv.silent);
    if (argv.source)
        config.source = String(argv.source);
    if (argv.tail)
        config.tail = String(argv.tail);
    if (argv.verbose)
        config.logOptions.verbose = Boolean(argv.verbose);
    return config;
}
/**
 * Reads a configuration file from the specified file path and returns the parsed configuration data.
 *
 * If the specified configuration file does not exist, it will attempt to read the default configuration file instead.
 * If there is an error reading either the specified or default configuration file, it will log the error and exit the process.
 *
 * @param fileName - The path to the configuration file to read.
 * @returns The parsed configuration data.
 */
function readConfigFile(fileName) {
    const configFileName = String(fileName);
    const configFileNameWithPath = node_path_1.default.join(node_path_1.default.dirname(process.cwd()), "..", configFileName);
    logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.debug(`Requested config file is '${configFileNameWithPath}'.`);
    let configData = null;
    if ((0, node_fs_1.existsSync)(configFileNameWithPath)) {
        logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.debug("File exists; reading...");
        try {
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.info(`Reading config file: '${configFileNameWithPath}'.`);
            configData = JSON.parse((0, node_fs_1.readFileSync)(configFileNameWithPath, "utf-8"));
        }
        catch (error) {
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(error);
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.warn("Error reading config file; see process.log for details.");
            process.exit(1);
        }
    }
    else {
        logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.debug("File does not existing; trying default.");
        try {
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.debug(`Reading default config file: '${node_path_1.default.join(__dirname, "..", constants_1.DEFAULT_DEFAULT_CONFIG_FILE_NAME)}'.`);
            configData = JSON.parse((0, node_fs_1.readFileSync)(node_path_1.default.join(__dirname, "..", constants_1.DEFAULT_DEFAULT_CONFIG_FILE_NAME), "utf-8"));
        }
        catch (error) {
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(error);
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.warn("Error reading config file; see process.log for details.");
            process.exit(1);
        }
    }
    logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.verbose(`Config is:\n${JSON.stringify(configData, null, 2)}`);
    logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.debug("Returning configuration data.");
    if (configData)
        return configData;
    logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error("Problem reading config file.");
    process.exit(1);
}
/**
 * Saves the current configuration to the specified configuration file.
 *
 * If the specified configuration file does not exist, it will be created.
 * If there is an error saving the configuration file, it will log the error and exit the process.
 *
 * @param argv - An object containing the command-line arguments, including the configuration file path.
 * @returns void
 */
function saveConfigToFile(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const configFile = node_path_1.default.join(process.cwd(), String(argv["config-file"]));
        let configData = globalThis.config;
        // Read existing config file if it exists
        if ((0, node_fs_1.existsSync)(configFile)) {
            try {
                const data = (0, node_fs_1.readFileSync)(configFile, "utf8");
                const existingConfig = JSON.parse(data);
                // If configs are equal, return early
                if (lodash_1.default.isEqual(existingConfig, configData)) {
                    return;
                }
                configData = Object.assign(Object.assign({}, existingConfig), configData);
            }
            catch (err) {
                // console.log("ERROR", err);
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(`Problem reading '${configFile}';  aborting!`);
                throw err;
            }
        }
        // Save configuration data to file
        if (yield (0, inquirer_1.saveConfig)()) {
            logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.verbose(`Writing config file to '${configFile}'`);
            try {
                (0, node_fs_1.writeFileSync)(configFile, JSON.stringify(configData, null, 2), "utf8");
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.verbose("Configuration file updated!");
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.verbose(JSON.stringify(configData, null, 2));
            }
            catch (err) {
                console.error({ err });
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(`Error saving configuration file to '${configFile}'`);
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(err);
                throw err;
            }
        }
    });
}
/**
 * Sets any missing keys in the provided configuration object to their default values.
 *
 * @param config - The configuration object to update.
 * @returns The updated configuration object.
 */
function setMissingKeysToDefault(config) {
    if (!config.debug)
        config.debug = false;
    if (!config.excludes)
        config.excludes = [];
    if (!config.force)
        config.force = false;
    if (!config.logOptions.silent)
        config.logOptions.silent = false;
    if (!config.logOptions.verbose)
        config.logOptions.verbose = false;
    return config;
}
/**
 * Validates the provided configuration object against a predefined schema.
 * If the configuration is invalid, logs the errors and exits the process.
 *
 * @param config - The configuration object to validate.
 * @returns void
 */
function validateConfig(config) {
    const res = schema_1.configSchema.safeParse(config);
    let configOk = true;
    if (!res.success) {
        const errors = res.error.errors;
        errors.map((error) => {
            if (error.code === "unrecognized_keys") {
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.warn(`${error.message}: ${error.keys.join()}; ignoring`);
            }
            else if (error.code === "invalid_type") {
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(`${error.path} is ${error.message.toLowerCase()}`);
                configOk = false;
            }
            else {
                logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.error(error);
                configOk = false;
            }
        });
    }
    const validConfig = config;
    if (validConfig.logOptions.verbose && validConfig.logOptions.silent) {
        logger_1.logger === null || logger_1.logger === void 0 ? void 0 : logger_1.logger.warn("Verbose and silent are mutually exclusive; ignoring both.");
        validConfig.logOptions.verbose = false;
        validConfig.logOptions.silent = false;
    }
    if (!configOk) {
        console.log("Error validating config file; see process.log for details");
        process.exit(1);
    }
}
//# sourceMappingURL=config.js.map