"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setLogLevel = exports.logLogLevels = exports.getLogLevels = exports.initializeLogger = exports.getFileTransportPath = exports.logger = void 0;
// biome-ignore lint/style/useNodejsImportProtocol: <explanation>
const fs_1 = require("fs");
const node_path_1 = __importDefault(require("node:path"));
const winston_1 = require("winston");
const { cli, combine, errors, printf, timestamp } = winston_1.format;
const filename = node_path_1.default.join(process.cwd(), "bruno-doc.log");
if ((0, fs_1.existsSync)(filename))
    (0, fs_1.unlinkSync)(filename);
/**
 * Closes all transports associated with the logger.
 * This function is called when the process is about to exit, ensuring that any open file transports are properly closed.
 */
const closeTransports = () => {
    if (!exports.logger)
        return;
    for (const transport of exports.logger.transports) {
        if (typeof transport.close === "function") {
            transport.close();
        }
    }
};
/**
 * A console transport for the logger that logs errors with their stack traces.
 * This transport is configured to log only error-level messages.
 */
const consoleTransport = new winston_1.transports.Console({
    format: combine(cli(), errors({ stack: true })),
    level: "info",
});
// TODO: Add documentation
// TODO: Add unit test
const createLoggerInstance = () => __awaiter(void 0, void 0, void 0, function* () {
    const stackTrace = yield import("stack-trace");
    const createDebugFormat = (stackTrace) => {
        return printf((info) => {
            const { level, message, timestamp } = info;
            const trace = stackTrace.get();
            const caller = trace[21];
            const fileName = caller.getFileName()
                ? node_path_1.default.basename(caller.getFileName())
                : " ";
            const lineNumber = caller.getLineNumber() ? caller.getLineNumber() : " ";
            const moduleInfo = `${fileName}:${lineNumber}`;
            return `${timestamp} [${level.toUpperCase()}] [${moduleInfo}] : ${message}`;
        });
    };
    const createFileTransport = (filename, stackTrace) => {
        const fileTransport = new winston_1.transports.File({
            filename,
            format: combine(timestamp(), createDebugFormat(stackTrace)),
            level: process.env.LOG_LEVEL || "verbose",
        });
        return fileTransport;
    };
    const fileTransport = createFileTransport(filename, stackTrace);
    exports.logger = (0, winston_1.createLogger)({
        transports: [consoleTransport, fileTransport],
    });
});
// TODO: Add documentation
const getFileTransportPath = () => {
    if (!exports.logger)
        return;
    const fileTransport = exports.logger.transports.find((transport) => transport.constructor.name === "File");
    if (fileTransport)
        return node_path_1.default.resolve(fileTransport.filename);
};
exports.getFileTransportPath = getFileTransportPath;
const initializeLogger = () => __awaiter(void 0, void 0, void 0, function* () {
    if (exports.logger)
        return;
    yield createLoggerInstance();
    process.on("exit", closeTransports);
    process.on("SIGINT", () => {
        closeTransports();
        process.exit(0);
    });
});
exports.initializeLogger = initializeLogger;
/**
 * Gets the log levels for all transports associated with the logger.
 * @returns An array of objects, where each object has a key-value pair representing the transport name and its log level.
 */
const getLogLevels = () => {
    if (!exports.logger)
        return [];
    const logLevels = [];
    for (const transport of exports.logger.transports) {
        logLevels.push({
            [transport.constructor.name]: transport.level || "unknown",
        });
    }
    return logLevels;
};
exports.getLogLevels = getLogLevels;
/**
 * Logs the current log levels for all transports associated with the logger.
 * This function iterates over the transports and logs the log level for each transport.
 */
const logLogLevels = () => {
    if (!exports.logger)
        return;
    for (const transport of exports.logger.transports) {
        exports.logger.verbose(`Level log for ${transport.constructor.name} transport is '${transport.level}'.`);
    }
};
exports.logLogLevels = logLogLevels;
/**
 * Sets the log level for the File transport of the logger.
 * @param level - The new log level to set for the File transport.
 */
const setLogLevel = (level) => {
    if (!exports.logger)
        return;
    exports.logger.verbose(`Setting log level for File transport to '${level}'.`);
    const fileTransport = exports.logger.transports.find((transport) => transport.constructor.name === "FileTransport");
    if (fileTransport)
        fileTransport.level = level;
    (0, exports.logLogLevels)();
};
exports.setLogLevel = setLogLevel;
(() => __awaiter(void 0, void 0, void 0, function* () {
    const stackTrace = yield import("stack-trace");
}))();
//# sourceMappingURL=logger.js.map